"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "grouped-rolemethods",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            FunctionDeclaration(node) {
                const dciContext = (0, DCIRuleHelpers_1.isContext)(node);
                if (!dciContext)
                    return;
                const nodes = node.body.body;
                // TODO: Should only RoleMethod functions be allowed in top-level?
                const roleMethods = new Map(nodes.map((node) => [
                    node,
                    node.type == ast_spec_1.AST_NODE_TYPES.FunctionDeclaration
                        ? dciContext.roleMethodCall(node.id?.name)
                        : null,
                ]));
                const rolePos = new Set();
                let currentRole = "";
                let lastNode;
                for (const [node, rm] of roleMethods) {
                    if (!rm) {
                        // A non-RoleMethod expression, cannot be placed between RoleMethods.
                        if (rolePos.size > 0) {
                            lastNode = node;
                        }
                    }
                    else if (lastNode) {
                        context.report({
                            messageId: "mixed",
                            node: lastNode,
                        });
                        lastNode = null;
                    }
                    else {
                        if (rm.role != currentRole) {
                            if (rolePos.has(rm.role)) {
                                context.report({
                                    messageId: "unordered",
                                    node,
                                });
                            }
                            else {
                                rolePos.add(rm.role);
                                currentRole = rm.role;
                            }
                        }
                    }
                }
            },
        });
    },
    meta: {
        docs: {
            description: "RoleMethods belonging to a Role must come after one another, they cannot be mixed with other Role's methods.",
            recommended: "error",
        },
        messages: {
            unordered: "RoleMethods belonging to a Role must come after one another, they cannot be mixed with other Role's methods.",
            mixed: "Statements and expressions cannot be placed between RoleMethods, only before or after.",
        },
        type: "problem",
        schema: [],
    },
    defaultOptions: [],
});
