"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
const allowedLiteralTypes = [
    ast_spec_1.AST_NODE_TYPES.TSTypeLiteral,
    ast_spec_1.AST_NODE_TYPES.TSNumberKeyword,
    ast_spec_1.AST_NODE_TYPES.TSStringKeyword,
    ast_spec_1.AST_NODE_TYPES.TSBooleanKeyword,
    ast_spec_1.AST_NODE_TYPES.TSBigIntKeyword,
    ast_spec_1.AST_NODE_TYPES.TSArrayType,
];
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "literal-role-contracts",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            FunctionDeclaration(node) {
                if (!(0, DCIRuleHelpers_1.isContext)(node))
                    return;
                for (const param of node.params)
                    if (!allowedLiteralTypes.includes(param.typeAnnotation?.typeAnnotation.type ?? "")) {
                        context.report({
                            loc: param.loc,
                            messageId: "literal",
                        });
                    }
            },
        });
    },
    meta: {
        docs: {
            description: "Role interfaces must be declared using an object type, array or primitive type. More info: https://www.typescriptlang.org/docs/handbook/2/everyday-types.html#object-types",
            recommended: "error",
        },
        messages: {
            literal: "Role interfaces must be declared using an object type, array or primitive type. More info: https://www.typescriptlang.org/docs/handbook/2/everyday-types.html#object-types",
        },
        type: "problem",
        schema: [],
    },
    defaultOptions: [],
});
