"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
const ast_spec_1 = require("@typescript-eslint/types/dist/generated/ast-spec");
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "private-role-access",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            CallExpression(callExpr) {
                const dciContext = (0, DCIRuleHelpers_1.isInContext)();
                if (!dciContext)
                    return;
                const currentFunc = (0, DCIRuleHelpers_1.currentFunction)();
                const checkAllowedCall = (call, isContractCall) => {
                    const caller = dciContext.roleMethodCall(currentFunc?.id?.name);
                    //console.log("Check call:", call);
                    // Within same Role is always allowed
                    if (caller && caller.role == call.role)
                        return;
                    if (call.isPrivate) {
                        context.report({
                            node: callExpr,
                            messageId: isContractCall ? "privateContractCall" : "privateCall",
                        });
                    }
                };
                switch (callExpr.callee.type) {
                    case ast_spec_1.AST_NODE_TYPES.Identifier: {
                        const calledRoleMethod = dciContext.roleMethodCall(callExpr.callee.name);
                        if (calledRoleMethod)
                            checkAllowedCall(calledRoleMethod, false);
                        break;
                    }
                    case ast_spec_1.AST_NODE_TYPES.MemberExpression: {
                        const member = callExpr.callee;
                        if (member.object.type == ast_spec_1.AST_NODE_TYPES.Identifier &&
                            member.property.type == ast_spec_1.AST_NODE_TYPES.Identifier) {
                            if (dciContext.roles.has(member.object.name)) {
                                checkAllowedCall({
                                    role: member.object.name,
                                    method: member.property.name,
                                    isPrivate: true,
                                }, true);
                            }
                        }
                        break;
                    }
                }
            },
        });
    },
    meta: {
        docs: {
            description: "Call to private RoleMethod outside its Role.",
            recommended: "error",
        },
        messages: {
            privateCall: "Call to private RoleMethod outside its Role.",
            privateContractCall: "Call to Role contract method outside its Role.",
        },
        type: "problem",
        schema: [],
    },
    defaultOptions: [],
});
