"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DCIRuleHelpers_1 = require("../DCIRuleHelpers");
exports.default = (0, DCIRuleHelpers_1.createRule)({
    name: "uppercase-roles",
    create(context) {
        return (0, DCIRuleHelpers_1.contextRules)(context, {
            FunctionDeclaration(node) {
                const dciContext = (0, DCIRuleHelpers_1.isContext)(node);
                if (!dciContext)
                    return;
                for (const [roleName, role] of dciContext.roles) {
                    if (roleName !== roleName.toLocaleUpperCase()) {
                        context.report({
                            node: role.id,
                            messageId: "notUppercase",
                        });
                    }
                }
            },
        });
    },
    meta: {
        docs: {
            description: "A Role name should be uppercase, to make it easy to identity in the code.",
            recommended: "warn",
        },
        messages: {
            notUppercase: "A Role name should be uppercase, to make it easy to identity in the code.",
        },
        type: "suggestion",
        schema: [],
    },
    defaultOptions: [],
});
